;ͻ
; MOUDLE	    C_SCAN.ASM 				       
;Ķ
; PURPOSE	    Scanner interface program			       
;Ķ
; DESCRIPTION	    Provides a consistant interface for APPS	       
;Ķ
; VERSION RELEASE  1.00					       
;Ķ
; AUTHOR	    RONG-JUNN WANG				       
;Ķ
; WALKTHROUGH	   						       
;Ķ
; MODIFICATION    						       
;ͼ

title	Scanner Driver

.xlist
	include scanner.inc
.list

; CALL procedure
;	call_buffer	scanner_interface_struc <>
;
;	lea	bx, call_buffer
;	mov	[bx].scanner_command, @InitiateScanner
;	call	_scanner_interface
;	cmp	[bx].scan_return, 0
;	jnz	error
;
;	mov	[bx].scanner_command, @SetDataType
;	mov	[bx].data_type, @ColorRGBRGB
;	call	_scanner_interface
;
;	:
;	:

code	segment public 'CODE'
	assume	cs:code, ds:code

;ͻ
; Variables							       
;ͼ

EVEN
i_am	call_parm, DWORD		; to save calling DS:BX
i_am	handle, WORD, -1		; HPSCAN driver handle

i_label scanner_procs, WORD
	dw	$Initiate_Scanner	; check scanner driver
	dw	$Reset_Scanner		; reset SCANNER (EscE)
	dw	$Start_Scanner		; start scanner (Esc*f0s)
	dw	$Set_Output_Data_Type	; output data type (Esc*a#T)
	dw	$Set_X_Y_Resolution	; set x & y resolutions
					; Esc*a#R for X, Esc*a#S for Y
	dw	$Set_X_Y_Position	; set x & y beginning position
					; Esc*f#X for X, Esc*f#Y for Y
					; by pixels
	dw	$Set_X_Y_Extent 	; set x width, y height by pixels
					; Esc*f#P for X, Esc*f#Q for Y
	dw	$Set_Image_Style	; set inverse image (Esc*a#I)
	dw	$Read_Scanner_Data	; read data from scanner
	dw	$Set_Data_Width 	; set data width (Esc*a#G)
	dw	$Inquire_Scan_Info	; read image informations
					; pixels per scan line - Esc*s1024E
					; bytes per scan line  - Esc*s1025E
					; number of scan lines - Esc*s1026E
@ScannerMaximumCommand	EQU	($-scanner_procs-2)/2

;=========================================

i_label dpi_tables, WORD
	dw	dpi_75
	dw	dpi_100
	dw	dpi_150
	dw	dpi_200
	dw	dpi_300

;ͻ
;								       
;ͼ

i_am	reset_code, BYTE, <1bh, 'E'>

i_am	data_width, BYTE, <1bh, '*a8G'>
@DataWidthLeng	EQU	$-data_width

i_am	start_scan, BYTE, <1bh, '*f0S'>
@StartSize	EQU	$-start_scan

;=======================================================================

i_am	esc_code, BYTE, <1bh, '*'>
i_am	cmd_put, 10h, 0

i_am	image_data, BYTE, <1bh, '*s102'>
i_am	image_code, BYTE, <'4E'>
i_am	pixel_value, 8, 0

;=========================================

i_am	dpi_75,  BYTE, <'75', 0>
i_am	dpi_100, BYTE, <'100', 0>
i_am	dpi_150, BYTE, <'150', 0>
i_am	dpi_200, BYTE, <'200', 0>
i_am	dpi_300, BYTE, <'300', 0>

;ͻ
; 1. Open scanner driver					       
; 2. Must be "character device"                                       
; 3. Set to binary mode for driver compatibility		       
; 4. Save handle for read/write				       
;ͼ
; i/p:
;	None
; o/p:
;	Scan_Return filed == 0: success
;			  != 0: failure

i_am	hp_driver, BYTE, <'HPSCAN', 0>

entry	$initiate_Scanner
	mov	ax, cs			;
	mov	ds, ax			; DS=ES=CS
	mov	es, ax			;

	lea	dx, hp_driver		;
	mov	ax, 3d42h		; open device
	int	21h			;
	jc	@f			; error

	mov	bx, ax			;
	mov	ax, 4400h		; get status to check whether or
	int	21h			; not a character device?
	test	dl, 80h 		;
	jz	close_quit		; no

	mov	ax, 4401h		;
	or	dl, 00100000B		; set to binary mode
	xor	dh, dh			;
	int	21h			;

	mov	handle, bx		; save handle for read/write
	xor	al, al			; success

exit_scan:
	lds	bx, call_parm		; get back the call in parameter@
	mov	[bx].scan_return, al	; fill return code
	ret

close_quit:
	mov	ah, 3eh 		;
	int	21h			; close before exit

@@:
	mov	al, -1			; failure
	jmp	exit_scan

;ͻ
; Reset the scanner driver. We recomment call every times after       
; scanner operations are done. 				       
;ͼ
; i/p:
;	None
; o/p:
;	None

entry	$Reset_Scanner
	push	cs			;
	pop	ds			; DS=CS

	mov	cx, 2			; length of command (EscE)
	lea	dx, reset_code		; command buffer
	call	_write_scanner		; write out
	ret

;ͻ
; Start up scanner						       
;ͼ
; i/p:
;	None
; o/p:
;	None

entry	$Start_Scanner
	push	cs			;
	pop	ds			; DS=CS

	lea	dx, start_scan		; command buffer
	mov	cx, @StartSize		; length of command (Esc*f0S)
	call	_write_scanner		; write out
	ret

;ͻ
; Specify the data should be B/W, Gray, Color, ......		       
;ͼ
; i/p:
;	DATA_TYPE field is prepared. The value of field is one of
;	definitions in SCAN.INC

entry	$Set_Output_Data_Type
	mov	bl, [bx].data_type	; BL = type of output data from scanner

	push	cs			;
	pop	ds			; DS=CS

	mov	ah, @SetOutputDataType	; Termination code of command
	call	_set_command		; prepare command string and write to
					; scanner
	ret

;ͻ
; Set resolution of Primary and Secondary direction		       
;ͼ
; i/p:
;	x_ppi, y_ppi are prepared. Both resolutions that provided by scanner
;	are 75, 100, 150, 200, 300 DPI. We use the index values to stand for
;	resolution strings.

entry	$Set_X_Y_Resolution
	push	[bx].y_ppi		; save Y-resolution index
	mov	si, [bx].x_ppi		; get index
	push	cs			;
	pop	ds			; DS=CS

	mov	al, @SetXResolution	; termination code
	call	myself

	pop	si			; get Y-resolution index
	mov	al, @SetYResolution	; termination code

myself:
	mov	si, dpi_tables [si]	; get
	call	_resolution_command	; put command code, string and terminat-
					; ion code, then write to sacnner
	ret

;ͻ
; Set X, Y origin						       
;ͼ
; i/p:
;	x_location, y_location are prepared (Unit: Device Pixels)

entry	$Set_X_Y_Position
	push	[bx].y_location 	; save y origin

	mov	dx, [bx].x_location	; x origin
	push	cs			;
	pop	ds			; DS=CS
	mov	al, @XTerminator	; termination code
	call	_binary_command 	; convert to ascii, prepare command
					; string, then write to scanner

	pop	dx			; y origin

	mov	al, @YTerminator	; termination code
	call	_binary_command 	; convert to ascii, prepare command
					; string, then write to scanner
	ret

;ͻ
; Set scanning range in pixels.				       
;ͼ
; i/p:
;	x_extent, y_extent are prepared. (width of primary direction in pixels,
;	height of secondary direction in pixels)

entry	$Set_X_Y_Extent
	push	[bx].y_extent		; y height

	mov	dx, [bx].x_extent	; x width
	push	cs			;
	pop	ds			; DS=CS
	mov	al, @PTerminator	; termination code
	call	_binary_command 	; convert to ascii, prepare command
					; string, then write to scanner

	pop	dx			; y height

	mov	al, @QTerminator	; termination code
	call	_binary_command 	; convert to ascii, prepare command
					; string, then write to scanner
	ret

;ͻ
; Set Inverse image or not.					       
;ͼ
; i/p:
;	inverse_flag is prepared. The values are defined in SCAN.INC

entry	$Set_Image_Style
	mov	bl, [bx].inverse_flag	; get style
	push	cs			;
	pop	ds			; DS=CS
	mov	ah, @SetInverseImage	; termination code
	call	_set_command		; prepare command string and write to
					; scanner
	ret

;ͻ
; Set how many bits per pixels.				       
;ͼ
; i/p:
;	data_width_def is prepared. The values are defined in SCAN.INC

entry	$Set_Data_Width
	mov	al, [bx].data_width_def ; output data width ('1', '4', or '8')
	push	cs			;
	pop	ds			; DS=CS
	mov	cmd_put, 'a'            ; leading command code
	mov	cmd_put [1], al 	; data width
	mov	cmd_put [2], @DataWidth ; termination code
	mov	cx, @DataWidthLeng	; length of command
	call	_write_command		; write to scanner
	ret

;ͻ
; Get pixels per scan line, bytes per scan line and number of scan    
; lines.							       
;ͼ

entry	$Inquire_Scan_Info
	push	cs			;
	pop	ds			; DS=CS

	mov	image_code, '4'         ; inquire command code "102'4'"
	call	_get_inquire_data	; get the value
	push	ax			; save readed value, pixels per scan
					; line

	mov	image_code, '5'         ; "1025", bytes per scan line
	call	_get_inquire_data	; get the value
	push	ax			; save the value

	mov	image_code, '6'         ; "1026", number of scan lines
	call	_get_inquire_data	; get the value

	lds	bx, call_parm		; DS:BX = call in parameter block@
	mov	[bx].lines_scan, ax	; number of scan lines
	pop	[bx].line_bytes 	; bytes per scan line
	pop	[bx].line_pixels	; pixels per scan line
	ret

;ͻ
; Read data from scanner to specific buffer.			       
;ͼ
; i/p:
;	buffer_leng, buffer@ are prepared.

	assume	cs:code, ds:nothing

entry	$Read_Scanner_Data
	mov	cx, [bx].buffer_leng	; number of bytes to read
	lds	dx, [bx].buffer@	; buffer@
	mov	bx, handle		; scanner handle
	mov	ah, 3fh 		; read
	int	21h
	ret

;ͻ
;								       
;ͼ

	assume	cs:code, ds:code

;ͻ
; Convert ascii to binary.					       
;ͼ
; i/p:
;	None
; o/p:
;	AX = binary value

entry	_ascii_to_binary
	lea	si, pixel_value 	; ascii string that returned from
					; scanner driver

	xor	cx, cx			; reset accumulator

@@:
	lodsb				; get a byte
	cmp	al, @InquireReturnValueTC
					; is terminator?
	jz	@f			; yes
	sub	al, '0'                 ; to binary (decimal system)
	cbw				; to word

	mov	bx, cx			;
	shl	cx, 1			;
	shl	cx, 1			; CX * 10 + AX
	add	cx, bx			;
	shl	cx, 1			;
	add	cx, ax			;
	jmp	@b
@@:
	mov	ax, cx
	ret

;ͻ
; Convert binary to ASCII, prepare the command string, then write     
; to scanner.							       
;ͼ
; i/p:
;	DX = binary value
;	AL = termination code

entry	_binary_command
	lea	di, cmd_put		; command string@
	push	ax			; save termination code
	mov	al, @SetDeviceCommand	; leading command code
	stosb				; put

	mov	ax, dx			;
	mov	bp, 1000		;
	xor	cx, cx			;
	call	div_put 		;
	mov	bp, 100 		; convert binary to ASCII and put them
	call	div_put 		; to command buffer
	mov	bp, 10			;
	call	div_put 		;
	add	al, '0'                 ;
	stosb				;

	inc	cx			; length of command string + 1
	add	cx, 4			; plus leading code, .. etc.

	pop	ax			; termination code
	stosb				; put
	call	_write_command		; write to scanner
	ret

;ͻ
; check and divide, then convert to ASCII			       
;ͼ
; i/p:
;	AX = dividend
;	BP = divisor, AX/BP
; o/p:
;	AX = remainder

entry	div_put
	mov	dx, ax			; save value
	cmp	ax, bp			; value >= divisor?
	jae	div_now 		; yes, then divide it
	jcxz	skip_it 		; first character?
	mov	al, '0'                 ; no, insert '0'
	jmp	put_it
div_now:
	xor	dx, dx			;
	div	bp			;
	or	al, al			; divide
	jnz	@f			;
	jcxz	skip_it 		;
@@:
	add	al, '0'
put_it:
	stosb
	inc	cx
skip_it:
	mov	ax, dx			; remainder
	ret

;ͻ
; Put asciiz to command string 				       
;ͼ
; i/p:
;	DS:SI = # value ASCII pointer
;	AL = termination code

entry	_resolution_command
	push	ax			; save terminator
	lea	di, cmd_put		;
	mov	al, @SetScannerCommand	; put leading code
	stosb				;
	mov	cx, 3			; length
	call	_put_until_null 	; put asciiz
	inc	cx			; include leading code
	pop	ax			; terminator
	stosb				; put
	call	_write_command		; write to scanner
	ret

;ͻ
; Put one character command code to command string, then write out    
;ͼ
; i/p:
;	BL = parameter
;	AH = terminator

entry	_set_command
	lea	di, cmd_put		; buffer
	mov	al, @SetScannerCommand	; leading code
	stosb				; put
	mov	al, bl			; parameter
	stosb
	mov	al, ah			; terminator
	stosb
	mov	cx, 5			; length
	call	_write_command		; write to scanner
	ret

;ͻ
; Write data to scanner driver 				       
;ͼ

entry	_write_command
	lea	dx, esc_code

entry	_write_scanner
	mov	bx, handle
	mov	ah, 40h
	int	21h
	ret

;ͻ
; Inquire data from scanner					       
;ͼ

entry	_get_inquire_data
	lea	dx, image_data		; command string buffer (also is inquire
					; string buffer)
	mov	image_code [1], 'E'     ; command code
	mov	cx, 8			; length
	call	_write_scanner		; write to scanner
	jc	@f
	call	_read_command		; read from scanner to buffer
	jc	@f
	call	_ascii_to_binary	; convert ascii to binary
@@:
	ret

;ͻ
; Read inquire data from scanner				       
;ͼ

entry	_read_command
	mov	cx, 10h
	mov	bx, handle
	mov	ah, 3fh
	int	21h
	ret

;ͻ
; move ASCIIZ to buffer (not included NULL)			       
;ͼ
; i/p:
;	DS:SI = data@
;	ES:DI = buffer@
; o/p:
;	CX will be accumulated

entry	_put_until_null
	lodsb
	or	al, al
	jz	@f
	stosb
	inc	cx
	jmp	_put_until_null
@@:
	ret

;ͻ
;								       
;ͼ

	assume	cs:code, ds:nothing

i_proc	_scanner_interface, FAR
	cmp	[bx].scanner_command, @ScannerMaximumCommand
	ja	@f
	push_regs	<ax, bx, cx, dx, si, di, bp, ds, es>
	mov	ax, cs
	mov	es, ax
	mov	call_parm_off, bx
	mov	call_parm_seg, ds
	mov	si, [bx].scanner_command
	shl	si, 1
	call	scanner_procs [si]
	pop_regs	<es, ds, bp, di, si, dx, cx, bx, ax>
@@:
	ret
_scanner_interface	endp

code	ends
	end
