
include macro.inc

;==================================
;=				  =
;=    System Informations File	  =
;=				  =
;==================================

file_spec_struc 	struc
	video_file	db	13 dup (0)
	video_mode_id	db	?
	scanner_file	db	13 dup (0)
;	tga_dir 	db	68 dup (0)
;	tiff_dir	db	68 dup (0)
file_spec_struc 	ends

@FileEntry		EQU	10h

file_header		struc
	file_ID 	db	'RJW.'
	file_size	dw	?
	file_depend	db	10 dup (?)
file_header		ends

scanner_file_s		struc
			db	6 dup (0)
	max_pixels	dw	?	; 2550 for A4
	max_lines	dw	?	; 3300 for A4
scanner_file_s		ends

video_file_s		struc
			db	6 dup (0)
	mode_number	dw	?	; number of modes
	mode_descriptor dw	?	; text of modes
video_file_s		ends

;======= TGA file format ==========

tga_file		struc
	tga_id		db	0
	tga_color_map	db	0
	tga_type	db	2
	tga_unknown	db	5 dup (0)
	tga_x_origin	dw	0
	tga_y_origin	dw	0
	tga_width	dw	640	; by pixels
	tga_high	dw	480	; by pixels
	tga_b_p_pixel	db	24	;
	tga_b_attr	db	0
tga_file		ends

;========= Interface of draw routine ===========

; RGB_Type

;	@RGBRGBHiC15	EQU	0	; HiColor (TGA)
;	@RGBRGBXGA16	EQU	1	; XGA
;	@RGBRGB24	EQU	2	; 8-bit R, G & B  (PP, TIFF)
;	@RRGGBB24	EQU	3	; 8-bit R..G..B.. (Hsc, TIFF)
;	@BGRBGR24	EQU	4	; 8-bit B..G..R.. (Hsc, TGA)

; Draw_Direct

	@UpLeftDown	EQU	00000001B
	@UpLeftUp	EQU	00000010B

; bits per Pixel

@Pixels24		EQU	24
@Pixels16		EQU	16

; draw method

@Replace		EQU	0
@Xor			EQU	2
@Or			EQU	4
@And			EQU	6

pic_info		struc
	video_command	dw	?
	Pixels_Buffer	dd	?	; number of pixels in buffer
	Pixels_Per_Line dw	?	; number of pines in buffer
	Pixels_Lines	dw	?	; number of pixels per line
	X_origin	dw	?	; x axis origin
	Y_origin	dw	?	; y axis origin
	Draw_Pixels	dw	?	; how many pixels to draw
	Draw_Lines	dw	?	; how many lines to draw
;	RGB_type	dw	@RGBRGBHiC15
;					; RGB style of buffer
	Draw_Method	dw	@Replace
	Draw_Direct	db	@UpLeftDown
					; showing direction
pic_info		ends

Left_Offset		EQU	Draw_method

Read_Write_Handle	EQU	x_origin

Pixels_Buffer_Off	EQU	word ptr Pixels_Buffer
Pixels_Buffer_Seg	EQU	word ptr Pixels_Buffer + 2
Modes_Buffer		EQU	Pixels_Buffer

Mode_Selected		EQU	pixels_per_line
Mode_View_Menu		EQU	pixels_Lines

@ModeMenu		EQU	0
@ModeView		EQU	2

@HorizontalLine 	EQU	00000001B
@VerticalLine		EQU	00000010B

X_End			EQU	Pixels_Per_Line ;
Y_End			EQU	Pixels_Lines	; for draw line, paint
HV_Flag 		EQU	byte ptr Pixels_lines
Pattern 		EQU	byte ptr Pixels_buffer
Pattern_count		EQU	byte ptr pixels_buffer+3			       ; up to 3-bytes for 24-bit

@CharWidth		EQU	8
@CharHeight		EQU	16

char_Buffer		EQU	Pixels_Buffer
char_width		EQU	draw_pixels
char_height		EQU	draw_lines
character		EQU	byte ptr Pixels_buffer
char_attr		EQU	byte ptr pixels_per_line
char_zattr		EQU	byte ptr pixels_per_line + 1
	@NoZAttr	EQU	0

@PictureInfoSize	EQU	size pic_info

;==== mode selections =====

@ModeAvailable		EQU	0ffh
@ModeInvalid		EQU	0

@640x480x256		EQU	0
@800x600x256		EQU	1
@1024x768x256		EQU	2
@640x480x32k		EQU	3
@800x600x32k		EQU	4
@1024x768x32k		EQU	5	; not available
@640x480x64k		EQU	6	; XGA

@640x480xPixels 	EQU	640
@640x480xLines		EQU	480
@800x600xPixels 	EQU	800
@800x600xLines		EQU	600
@1024x768xPixels	EQU	1024
@1024x768xLines 	EQU	768

;==========================

@HiColor640		EQU	2Eh
@HiColor800		EQU	30h
@TextMode		EQU	3
@EGA16			EQU	10h
@VGA16			EQU	12h
@VGA256 		EQU	13h

;==========================

@MemoryToVideo		EQU	0
@PatternToVideo 	EQU	2
@LinePaint		EQU	4
@FramePaint		EQU	6
@ReadPixels		EQU	8
@ShowCharacter		EQU	10
@ShowString		EQU	12
@ShowBlockMessage	EQU	14

;==========================

@InquireVideoModes	EQU	0
	; i/p:
	;	DS:BX = parameter@

@VideoInitiate		EQU	1
	; i/p:
	;	DS:BX = parameter@
	;
	;	mode_selected must be prepared

@VideoRestore		EQU	2

@VideoSaveBuffer	EQU	3
	; i/p:
	;	DS:BX = parameter@
	;	read_write_handle: file handle

@VideoRestoreBuffer	EQU	4
	; i/p:
	;	DS:BX = parameter@
	;	read_write_handle: file handle

@VideoDraw		EQU	5
	; i/p:
	;	DS:BX = parameter@
	;	x_origin       : x coordination
	;	y_origin       : y coordination
	;	pixels_buffer  : pixels' buffer to draw
	;	draw_pixels    : number of pixels to draw
	;	draw_lines     : number of lines to draw
	;	pixels_per_line: pixels per record in buffer

;@VideoScrollUp 	 EQU	 6
	; i/p:
	;	DS:BX = parameter@
	;	x_origin       :
	;	y_origin       : (x, y) for upper left point
	;	draw_pixels    : width to scroll
	;	draw_lines     : lines to scroll

;@VideoScrollDown	 EQU	 7
	; i/p:
	;	DS:BX = parameter@
	;	x_origin       :
	;	y_origin       : (x, y) for upper left point
	;	draw_pixels    : width to scroll
	;	draw_lines     : lines to scroll

@VideoReadPixels	EQU	6
	; i/p:
	;	DS:BX = parameter@
	;	x_origin       : x coordination
	;	y_origin       : y coordination
	;	pixels_buffer  : buffer to save the data
	;	draw_pixels    : number of pixels to read
	;	draw_lines     : number of lines to read

@VideoDrawLine		EQU	7
	; i/p:
	;	DS:BX = parameter@
	;	x_origin       : x begining coordination
	;	y_origin       : y begining coordination
	;	draw_pixels    : number of pixels to draw
	;	hv_flag        : Horizontal or Vertical
	;	draw_method    : method to draw the line

@VideoDrawFrame 	EQU	8
	; i/p:
	;	DS:BX = parameter@
	;	x_origin       : x begining coordination
	;	y_origin       : y begining coordination
	;	x_end	       : x end coordination
	;	y_end	       : y end coordination
	;	draw_method    : method to draw the frame
	;	pattern        : the data to be filled (RGB 24/16/8)
	;	pattern_count  : number of pattern in byte

@VideoPaintBlock	EQU	9
	; i/p:
	;	DS:BX = parameter@
	;	x_origin       : x begining coordination
	;	y_origin       : y begining coordination
	;	draw_pixels    : number of pixels to draw
	;	draw_lines     : number of lines to draw
	;	pattern        : the data to be filled (RGB 24/16/8)
	;	pattern_count  : number of pattern in byte

@VideoShowCharacter	EQU	10
	;	DS:BX = parameter@
	;	x_origin       : x begining coordination
	;	y_origin       : y begining coordination
	;	char_width     : number of pixels to draw
	;	char_height    : number of lines to draw
	;	character      : the character to be displayed
	;	char_attr      : attribute of character

@VideoShowString	EQU	11
	;	DS:BX = parameter@
	;	char_buffer    : buffer of characters
	;	x_origin       : x begining coordination
	;	y_origin       : y begining coordination
	;	char_width     : number of pixels to draw
	;	char_height    : number of lines to draw
	;	char_attr      : attribute of character

@VideoShowBlock 	EQU	12
	;	DS:BX = parameter@
	;	char_buffer    : buffer of characters
	;	x_origin       : x begining coordination
	;	y_origin       : y begining coordination
	;	char_width     : number of pixels to draw
	;	char_height    : number of lines to draw
	;	char_attr      : attribute of character

@VideoDrawShadow	EQU	13
	;	DS:BX = parameter@
	;	x_origin       : x begining coordination
	;	y_origin       : y begining coordination
	;	draw_pixels    : number of pixels to draw
	;	draw_lines     : number of lines to draw

@VideoDrawSpecial	EQU	14
	; i/p:
	;	DS:BX = parameter@
	;	x_origin       : x coordination
	;	y_origin       : y coordination
	;	pixels_buffer  : pixels' buffer to draw
	;	draw_pixels    : number of pixels to draw
	;	draw_lines     : number of lines to draw
	;	pixels_per_line: pixels per record in buffer

;@VideoDrawOffset	 EQU	 15
	; i/p:
	;	DS:BX = parameter@
	;	x_origin       : x coordination
	;	y_origin       : y coordination
	;	pixels_buffer  : pixels' buffer to draw
	;	draw_pixels    : number of pixels to draw
	;	draw_lines     : number of lines to draw
	;	pixels_per_line: pixels per record in buffer
	;	left_offset    : number of pixels to skip

@VideoReadColors	EQU	15
	; i/p:
	;	DS:BX = parameter@
	;	x_origin       : x coordination
	;	y_origin       : y coordination
	;	pixels_buffer  : buffer to save the data
	;	draw_pixels    : number of pixels to read
	;	draw_lines     : number of lines to read

;==========================

video_depend_parms	struc
	x_width 	dw	?
	y_height	dw	?
	paint_routine	dw	?
video_depend_parms	ends


;==========================

@CrtMode		EQU	449h
@CrtColumns		EQU	44Ah
@CrtLength		EQU	44Ch
@CrtStart		EQU	44Eh

include mouse.inc
include menu.inc
include scan.inc
include mem.inc
include show.inc

